<?php

namespace QPMN\Partner\Admin;

use QPMN\Partner\Admin\Obj\Qpmn_Admin as ObjQpmn_Admin;
use QPMN\Partner\Qpmn_Install;
use QPMN\Partner\WC\QP_WP_Option_Account;

class Qpmn_Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $pages;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->pages = [
			Qpmn_Admin_Menu::ADMIN_MAIN_SLUG => 1,
			Qpmn_Admin_Menu::ADMIN_PRODUCT_SLUG => 1,
			Qpmn_Admin_Menu::ADMIN_ORDER_SLUG => 1,
			Qpmn_Admin_Menu::ADMIN_LOG_SLUG => 1,
		];
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		global $pagenow;
		$page = sanitize_text_field($_GET['page'] ?? null) ;
		if ($pagenow === 'admin.php' && $page && array_key_exists($page, $this->pages)) {
			wp_enqueue_style($this->plugin_name . '-qpmn-boostrap', plugin_dir_url(__FILE__) . 'css/bootstrap/qpmn-bootstrap-iso.min.css', array(), $this->version);
			wp_enqueue_style($this->plugin_name . '-font-awesome-4.7', plugin_dir_url(__FILE__) . '../Public/css/font-awesome/4.7.0/css/font-awesome.min.css', array(), $this->version);
		}
		if ($pagenow === 'admin.php' || $pagenow === 'post.php') {
			wp_enqueue_style($this->plugin_name . '-qpmn-boostrap', plugin_dir_url(__FILE__) . 'css/bootstrap/qpmn-bootstrap-iso.min.css', array(), $this->version);
			if (QPMN_ENV == 'DEBUG') {
				wp_enqueue_style($this->plugin_name . 'admin-css', plugin_dir_url(__FILE__) . 'css/qpmn-admin-iso.css', array(), $this->version);
			} else {
				wp_enqueue_style($this->plugin_name . 'admin-css', plugin_dir_url(__FILE__) . 'css/qpmn-admin-iso.min.css', array(), $this->version);
			}
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
		global $pagenow;
		$page = sanitize_text_field($_GET['page'] ?? '');
		$action = sanitize_text_field($_GET['action'] ?? '');

		if ($pagenow === 'admin.php' && $page && array_key_exists($page, $this->pages)) {
			//plugin admin pages

			wp_register_script($this->plugin_name . '-bootstrap', plugin_dir_url(__FILE__) . 'js/bootstrap/5.1.3/bootstrap.bundle.min.js', array('jquery'), $this->version, false);
			wp_enqueue_script($this->plugin_name . '-bootstrap');

			if (QPMN_ENV == 'DEBUG') {
				wp_register_script($this->plugin_name . '-vuejs', plugin_dir_url(__FILE__) . 'js/vue/2.6.14/vue.js', [], $this->version, false);
				wp_enqueue_script($this->plugin_name . '-vuejs');
			} else {
				wp_register_script($this->plugin_name . '-vuejs', plugin_dir_url(__FILE__) . 'js/vue/2.6.14/vue.min.js', [], $this->version, false);
				wp_enqueue_script($this->plugin_name . '-vuejs');
			}

			wp_register_script($this->plugin_name . '-vuex', plugin_dir_url(__FILE__) . 'js/vuex/3.6.2/vuex.min.js', [$this->plugin_name . '-vuejs'], $this->version, false);
			wp_enqueue_script($this->plugin_name . '-vuex');

			wp_register_script($this->plugin_name . '-axios', plugin_dir_url(__FILE__) . 'js/axios/0.26.1/axios.min.js', [$this->plugin_name . '-vuejs'], $this->version, false);
			wp_enqueue_script($this->plugin_name . '-axios');

		}

		if ($pagenow === 'post.php' && $action && $action == 'edit') {
			//order page or plugin admin pages
			wp_register_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/qpmn-product-meta.js', array('jquery'), $this->version, false);
			wp_localize_script($this->plugin_name, 'qpmn_admin_post_obj', array(
				'ajax_url' 	=> rest_url(Qpmn_Install::PLUGIN_API_NAMESPACE),
				'nonce' 	=> wp_create_nonce('wp_rest'),
			));
			wp_enqueue_script($this->plugin_name);
		}
		if ($pagenow === 'admin.php') {
			wp_register_script($this->plugin_name . 'admin', plugin_dir_url(__FILE__) . 'js/qpmn-admin.js', array('jquery'), $this->version, false);
			wp_localize_script($this->plugin_name . 'admin', 'qpmn_admin_obj', ObjQpmn_Admin::get_obj());
			wp_enqueue_script($this->plugin_name . 'admin');
		}
	}

	public function init_admin()
	{

		if (empty(get_option(Qpmn_Install::ACCOUNT_VERIFIED))) {
			//setup page affected by this flag
			QP_WP_Option_Account::saveAndUpdate(false, Qpmn_Install::ACCOUNT_VERIFIED);
		}
	}

	public function add_style_attributes($html, $handle)
	{

		return $html;
	}
}
