const { __, _x, _n, sprintf } = wp.i18n;

const qpmn_axios_instance = axios.create({
    baseURL: qpmn_builder_obj.ajax_url,
    headers: {
        'X-WP-Nonce': qpmn_builder_obj.nonce
    }
});
const qpmn_personalize_product_mixin = {
    methods: {
        __: function(text) {
            console.log('__()');
            return __(text, 'qp-market-network');
        }
    }
};
const qpmn_personalize_product_store = new Vuex.Store({
    state: {
        showLoading: true,
        iframeSrc: '',
        designId: '',
        postId: '',
        thumbnail: '',
        siteUrl: '',
        nonce: qpmn_builder_obj.nonce,
    },
    mutations: {
        changeDesignId: function(state, payload) {
            if (!payload || payload === "" || isNaN(payload)) {
                //empty design id 
                jQuery('.single_add_to_cart_button').hide();
                jQuery('#qpmn-personalize-product').show();
            } else {
                jQuery('.single_add_to_cart_button').show();
                jQuery('#qpmn-personalize-product').hide();
            }
            state.designId = payload;
        },
        changeThumbnail: function(state, payload) {
            if (!payload || payload.length === 0) {
                return false;
            }
            state.thumbnail = payload;
            let imgUrl = payload;
            var existsELe = jQuery('.woocommerce-product-gallery__image.qpmn-personalize-thumbnail');

            if (existsELe.length) {
                //exists found
                existsELe.remove();
            }
            var targetDom = jQuery('.woocommerce-product-gallery__wrapper');
            var img = '<img src="' + payload + '" class="" alt="" title="" data-caption="" data-src="' + imgUrl + '"' +
                ' data-large_image_width="510" data-large_image_height="510" data-large_image="' + imgUrl + '" srcset="' + imgUrl + '">'
            var slide = '<div data-thumb="' + imgUrl + '" data-thumb-alt="text" class="woocommerce-product-gallery__image qpmn-personalize-thumbnail"><a href="' + imgUrl + '">' + img + '</a></div>';
            jQuery('.woocommerce-product-gallery__image--placeholder').hide();
            targetDom.prepend(slide);
        },
        changeShowLoading: function(state, payload) {
            state.showLoading = payload;
        },
        changeiFrameSrc: function(state, payload) {
            var newIframeSrc = payload || state.iframeSrc;
            var siteUrl = state.siteUrl;
            var newURL = new URL(newIframeSrc, siteUrl);

            //reset query string
            newURL.searchParams.set('state', state.nonce);
            newURL.searchParams.set('fromurl', window.location.href);

            state.iframeSrc = newURL.href;
        },
    },
    actions: {
        init: function(context, payload) {
            let eleProxyPost = jQuery('.product form.cart input[name="proxypost"]');
            var eleIframeSrc = jQuery('.product form.cart input[name="qp_iframe_src"]');
            let eleDesignId = jQuery('.product form.cart input[name="qp_design_id"]');
            let eleThumbnail = jQuery('.product form.cart input[name="qp_design_thumbnail"]');
            let eleUrl = jQuery('.product form.cart input[name="site_url"]');

            //trigger ui handle

            //get data form query string  
            const params = new URLSearchParams(window.location.search);
            let statusCode = params.get('statuscode') ?? null;
            let state = params.get('state') ?? null;

            let image = params.get('image_url') ?? null;

            //use form designID first
            let designId = eleDesignId.val() ?? null;
            if (!designId || designId.length < 0) {
                //existing design not found , try to use query string
                designId = params.get('designid') ?? null;

                if (designId && designId.length > 0) {
                    //builder design id found
                    if (context.state.nonce != state) {
                        //check nonce
                        //redirect to normal product page
                        window.location.href = window.location.origin + window.location.pathname;
                    }
                }
            }
            let thumbnail = eleThumbnail.val() ?? null;
            if (!thumbnail || thumbnail.length < 0) {
                thumbnail = image;
            }
            
            //assign data
            eleDesignId.val(designId);
            if (thumbnail && thumbnail.length) {
                eleThumbnail.val(thumbnail);
            }


            context.state.postId    = eleProxyPost.val();
            context.state.thumbnail = eleThumbnail.val();
            context.state.siteUrl   = eleUrl.val();

            context.commit('changeDesignId', designId);
            context.commit('changeThumbnail', context.state.thumbnail);

            //last step - prepare iframe src
            context.commit('changeiFrameSrc', eleIframeSrc.val());
            context.commit('changeShowLoading', false);
        },
    }
});

if (jQuery('#qpmn-personalize-product').length) {

    //function: control and determine buttons display
    //provide init data 
    //determine produjct is configurable
    const qpmn_personalize_product = new Vue({
        mixins: [qpmn_personalize_product_mixin],
        el: '#qpmn-personalize-product',
        store: qpmn_personalize_product_store,
        template: `
    <div class="qpmn-public qpmn-bootstrap" id='qpmn-personalize-product'>
        <div v-if="showLoading" class="btn">
            <div class="spinner-border" role="status">
                <span class="sr-only">...</span>
            </div>
        </div>
        <div v-if="!showLoading" class="qpmn-personalize-product-container">
            <qpmn-builder-custom-btn></qpmn-builder-custom-btn>
        </div>
    </div>
    `,
        computed: {
            showLoading: {
                set(val) {
                    qpmn_personalize_product_store.commit('changeShowLoading', val);
                },
                get() {
                    return qpmn_personalize_product_store.state.showLoading;
                }
            },
            isConfigurable: {
                get() {
                    return qpmn_personalize_product_store.state.isConfigurable;
                }
            },
        },
        mounted: function() {
            this.$nextTick(function() {
                qpmn_personalize_product_store.dispatch('init');
            });
        }
    });
}

Vue.component('qpmn-builder-custom-btn', {
    mixins: [qpmn_personalize_product_mixin],
    template: '<button v-on:click.stop.prevent="openbuilder">{{customize}}</button>',
    computed: {
        customize: function() {
            return this.__('Customize');
        }
    },
    methods: {
        openbuilder: function(event) {
            //open a specific window to display qpmn builder
            //prepare builder url with fromurl and state 
            window.open(qpmn_personalize_product_store.state.iframeSrc, 'qpmn-builder');
        }
    }
});

